@@ -46,6 +46,7 @@ extern int run_timer;
 
 #define GTT_GCONF "/apps/gnotime"
 
+
 /* ======================================================= */
 
 void
@@ -142,7 +143,7 @@ gtt_gconf_save (void)
 	SETBOOL ("/Display/ShowImportance", config_show_title_importance);
 	SETBOOL ("/Display/ShowStatus", config_show_title_status);
 
-	xpn = ctree_get_expander_state (global_ptw);
+	xpn = gtt_projects_tree_get_expander_state (projects_tree);
 	SETSTR ("/Display/ExpanderState", xpn);
 
 	/* ------------- */
@@ -201,7 +202,7 @@ gtt_gconf_save (void)
 		GSList *list= NULL;
 		for (i=0, w=0; -1< w; i++) 
 		{
-			w = ctree_get_col_width (global_ptw, i);
+			w = gtt_projects_tree_get_col_width (projects_tree, i);
 			if (0 > w) break;
 			list = g_slist_prepend (list, (gpointer) w);
 		}
@@ -396,6 +397,9 @@ gtt_gconf_load (void)
 	}
 
 	config_show_secs            = GETBOOL ("/Display/ShowSecs", FALSE);
+
+	prefs_set_show_secs ();
+
 	config_show_clist_titles    = GETBOOL ("/Display/ShowTableHeader", FALSE);
 	config_show_subprojects     = GETBOOL ("/Display/ShowSubProjects", TRUE);
 	config_show_statusbar       = GETBOOL ("/Display/ShowStatusbar", TRUE);
@@ -418,8 +422,8 @@ gtt_gconf_load (void)
 	config_show_title_urgency    = GETBOOL ("/Display/ShowUrgency", TRUE);
 	config_show_title_importance = GETBOOL ("/Display/ShowImportance", TRUE);
 	config_show_title_status     = GETBOOL ("/Display/ShowStatus", FALSE);
-	ctree_update_column_visibility (global_ptw);
 
+	prefs_update_projects_view ();
 
 	/* ------------ */
 	config_show_toolbar    = GETBOOL ("/Toolbar/ShowToolbar", TRUE);
@@ -463,7 +467,7 @@ gtt_gconf_load (void)
 			num = (int)(node->data);
 			if (-1 < num)
 			{
-				ctree_set_col_width (global_ptw, i, num);
+				gtt_projects_tree_set_col_width (projects_tree, i, num);
 			}
 		}
 	}
