@@ -22,18 +22,18 @@
 #include <glib.h>
 #include <guile/gh.h>
 #include <libguile.h>
+#include <libguile/backtrace.h>
 #include <limits.h>
 #include <stdio.h>
 #include <string.h>
 #include <libgnomevfs/gnome-vfs.h>
 
-#include <qof/qof.h>
+#include <qof.h>
 
 #include <locale.h>
 #include <monetary.h>
 
 #include "app.h"
-#include "ctree.h"
 #include "cur-proj.h"
 #include "gtt.h"
 #include "ghtml.h"
@@ -69,7 +69,7 @@ GttGhtml *ghtml_guile_global_hack = NULL;
 static SCM
 do_ret_did_query (GttGhtml *ghtml)
 {
-	return SCM_BOOL (ghtml->did_query);
+	return scm_from_bool (ghtml->did_query);
 }
 
 static SCM
@@ -88,7 +88,7 @@ reverse_list (SCM node_list)
 	SCM rc, node;
 	rc = SCM_EOL;
 
-	while (FALSE == SCM_NULLP(node_list))
+	while (!scm_is_null (node_list))
 	{
 		node = SCM_CAR (node_list);
 		rc = scm_cons (node, rc);
@@ -118,36 +118,33 @@ do_apply_based_on_type (GttGhtml *ghtml, SCM node,
              SCM (*ivl_func)(GttGhtml *, GttInterval *))
 {
 	/* Either a 'symbol or a "quoted string" */
-	if (SCM_SYMBOLP(node) || SCM_STRINGP (node))
+	if (scm_is_symbol(node) || scm_is_string (node))
 	{
 		SCM rc = SCM_EOL;
-		char *str = SCM_STRING_CHARS (node);
-		int len = SCM_STRING_LENGTH (node);
+		char *str = scm_to_locale_string (node);
+		int len = strlen (str);
 		if ((0<len) && str_func) rc = str_func (ghtml, str);
 		return rc;
 	}
 
 	/* If its a number, its in fact a pointer to the C struct. */
-	if (SCM_NUMBERP(node))
+	if (scm_is_number(node))
 	{
 		SCM rc = SCM_EOL;
 		switch (cur_type)
 		{
 			case GTT_PRJ: {
-				GttProject *prj = (GttProject *) scm_num2ulong (node,
-				             SCM_ARG1, "GnoTime::do-apply-based-on-type==project");
+				GttProject *prj = (GttProject *) scm_to_ulong (node);
 				if (prj_func) rc = prj_func (ghtml, prj);
 				break;
 			}
 			case GTT_TASK: {
-				GttTask *tsk = (GttTask *) scm_num2ulong (node,
-				             SCM_ARG1, "GnoTime::do-apply-based-on-type==task");
+				GttTask *tsk = (GttTask *) scm_to_ulong (node);
 				if (tsk_func) rc = tsk_func (ghtml, tsk);
 				break;
 			}
 			case GTT_IVL: {
-				GttInterval *ivl = (GttInterval *) scm_num2ulong (node,
-				             SCM_ARG1, "GnoTime::do-apply-based-on-type==interval");
+				GttInterval *ivl = (GttInterval *) scm_to_ulong (node);
 				if (ivl_func) rc = ivl_func (ghtml, ivl);
 				break;
 			}
@@ -159,7 +156,7 @@ do_apply_based_on_type (GttGhtml *ghtml, SCM node,
 	}
 
 	/* If its a list, then process the list */
-	if (SCM_CONSP(node))
+	if (scm_is_pair(node))
 	{
 		SCM rc = SCM_EOL;
 		SCM node_list = node;
@@ -168,14 +165,14 @@ do_apply_based_on_type (GttGhtml *ghtml, SCM node,
 		 * type.  If so, then strip off the label, and pass back
 		 * car to ourselves, and passing the corrected type.
 		 */
-		if (FALSE == SCM_NULLP(node))
+		if (!scm_is_null (node))
 		{
 			SCM type;
 			type = SCM_CDR (node);
-			if (SCM_SYMBOLP(type) || SCM_STRINGP (type))
+			if (scm_is_symbol(type) || scm_is_string (type))
 			{
 				cur_type = GTT_NONE;
-				char *buff = SCM_STRING_CHARS (type);
+				char *buff = scm_to_locale_string (type);
 
 				if ((!strncmp (buff, "gtt-project-ptr",15)) ||
 				    (!strncmp (buff, "gtt-project-list",16)))
@@ -203,7 +200,7 @@ do_apply_based_on_type (GttGhtml *ghtml, SCM node,
 		/* Otherwise, we have just a list. Walk that list,
 		 * apply recursively to it.
 		 */
-		while (FALSE == SCM_NULLP(node_list))
+		while (!scm_is_null (node_list))
 		{
 			SCM evl;
 			node = SCM_CAR (node_list);
@@ -211,7 +208,7 @@ do_apply_based_on_type (GttGhtml *ghtml, SCM node,
 			evl = do_apply_based_on_type (ghtml, node, cur_type,
 				               str_func, prj_func, tsk_func, ivl_func);
 
-			if (FALSE == SCM_NULLP (evl))
+			if (!scm_is_null (evl))
 			{
 				rc = scm_cons (evl, rc);
 			}
@@ -226,7 +223,7 @@ do_apply_based_on_type (GttGhtml *ghtml, SCM node,
 	}
 
 	/* If its a null list, do nothing */
-	if (SCM_NULLP (node))
+	if (scm_is_null (node))
 	{
 		return node;
 	}
@@ -284,7 +281,7 @@ kvp_cb (GttGhtml *ghtml, const char *key)
 	if (!val) return SCM_EOL;
 	str = kvp_value_get_string (val);
 	if (!str) return SCM_EOL;
-	return scm_mem2string (str, strlen (str));
+	return scm_from_locale_string (str);
 }
 
 static SCM
@@ -308,13 +305,13 @@ do_show_scm (GttGhtml *ghtml, SCM node)
 
 	/* Need to test for numbers first, since later tests
 	 * may core-dump guile-1.3.4 if the node was in fact a number. */
-	if (SCM_NUMBERP(node))
+	if (scm_is_number(node))
 	{
 		char buf[132];
 		double x;
 		long  ix;
 
-		x = scm_num2dbl (node, "GnoTime::do_show_scm");
+		x = scm_to_double (node);
 		ix = (long) x;
 
 		/* If the number is representable in 32 bits,
@@ -334,14 +331,14 @@ do_show_scm (GttGhtml *ghtml, SCM node)
 	}
 	else
 	/* either a 'symbol or a "quoted string" */
-	if (SCM_SYMBOLP(node) || SCM_STRINGP (node))
+	if (scm_is_symbol(node) || scm_is_string (node))
 	{
-		str = SCM_STRING_CHARS (node);
-		len = SCM_STRING_LENGTH (node);
+		str = scm_to_locale_string (node);
+		len = strlen (str);
 		if (0<len) (ghtml->write_stream) (ghtml, str, len, ghtml->user_data);
 	}
 	else
-	if (SCM_CONSP(node))
+	if (scm_is_pair(node))
 	{
 		SCM node_list = node;
 		do
@@ -350,19 +347,19 @@ do_show_scm (GttGhtml *ghtml, SCM node)
 			do_show_scm (ghtml, node);
 			node_list = SCM_CDR (node_list);
 		}
-		while (SCM_CONSP(node_list));
+		while (scm_is_pair(node_list));
 		do_show_scm (ghtml, node_list);
 	}
 	else
-	if (SCM_BOOLP(node))
+	if (scm_is_bool(node))
 	{
 		const char *str;
-		if (SCM_FALSEP(node)) str = _("False");
+		if (scm_is_false (node)) str = _("False");
 		else str = _("True");
 		(ghtml->write_stream) (ghtml, str, strlen(str), ghtml->user_data);
 	}
 	else
-	if (SCM_NULLP(node))
+	if (scm_is_null (node))
 	{
 		/* No op; maybe this should be a warning? */
 	}
@@ -398,10 +395,10 @@ static SCM
 do_ret_project (GttGhtml *ghtml, GttProject *prj)
 {
 	SCM node,rc;
-	rc = scm_ulong2num ((unsigned long) prj);
+	rc = scm_from_ulong ((unsigned long) prj);
 
 	/* Label the pointer with a type identifier */
-	node = scm_mem2string ("gtt-project-ptr", 15);
+	node = scm_from_locale_string ("gtt-project-ptr");
 	rc = scm_cons (rc, node);
 
 	return rc;
@@ -414,7 +411,7 @@ do_ret_project (GttGhtml *ghtml, GttProject *prj)
 static SCM
 do_ret_selected_project (GttGhtml *ghtml)
 {
-	GttProject *prj = ctree_get_focus_project (global_ptw);
+	GttProject *prj = gtt_projects_tree_get_selected_project (projects_tree);
 	return do_ret_project (ghtml, prj);
 }
 
@@ -478,14 +475,14 @@ static SCM
 do_include_file_scm (GttGhtml *ghtml, SCM node)
 {
 	/* either a 'symbol or a "quoted string" */
-	if (SCM_SYMBOLP(node) || SCM_STRINGP (node))
+	if (scm_is_symbol(node) || scm_is_string (node))
 	{
-		const char * filepath = SCM_STRING_CHARS (node);
+		const char * filepath = scm_to_locale_string (node);
 		filepath = gtt_ghtml_resolve_path(filepath, ghtml->ref_path);
 		gtt_ghtml_display (ghtml, filepath, NULL);
 	}
 	else
-	if (SCM_CONSP(node))
+	if (scm_is_pair(node))
 	{
 		SCM node_list = node;
 		do
@@ -494,11 +491,11 @@ do_include_file_scm (GttGhtml *ghtml, SCM node)
 			do_include_file_scm (ghtml, node);
 			node_list = SCM_CDR (node_list);
 		}
-		while (SCM_CONSP(node_list));
+		while (scm_is_pair(node_list));
 		do_include_file_scm (ghtml, node_list);
 	}
 	else
-	if (SCM_NULLP(node))
+	if (scm_is_null (node))
 	{
 		/* No op; maybe this should be a warning? */
 	}
@@ -541,13 +538,13 @@ g_list_to_scm (GList * gplist, const char * type)
 		/* Walk backwards, creating a scheme list */
 		for (n= gplist; n; n=n->prev)
 		{
-			node = scm_ulong2num ((unsigned long) n->data);
+			node = scm_from_ulong ((unsigned long) n->data);
 			rc = scm_cons (node, rc);
 		}
 	}
 
 	/* Prepend type label */
-	node = scm_mem2string (type, strlen (type));
+	node = scm_from_locale_string (type);
 	rc = scm_cons (rc, node);
 
 	return rc;
@@ -588,7 +585,7 @@ do_ret_project_list (GttGhtml *ghtml, GList *proj_list)
 			rc = scm_cons (node, rc);
 		}
 #endif
-		node = scm_ulong2num ((unsigned long) prj);
+		node = scm_from_ulong ((unsigned long) prj);
 		rc = scm_cons (node, rc);
 	}
 	return rc;
@@ -674,9 +671,9 @@ do_ret_tasks (GttGhtml *ghtml, GttProject *prj)
 	for (n= task_list; n; n=n->prev)
 	{
 		GttTask *tsk = n->data;
-      SCM node;
+		SCM node;
 
-		node = scm_ulong2num ((unsigned long) tsk);
+		node = scm_from_ulong ((unsigned long) tsk);
 		rc = scm_cons (node, rc);
 	}
 	return rc;
@@ -719,7 +716,7 @@ do_ret_intervals (GttGhtml *ghtml, GttTask *tsk)
 		GttInterval *ivl = n->data;
 		SCM node;
 
-		node = scm_ulong2num ((unsigned long) ivl);
+		node = scm_from_ulong ((unsigned long) ivl);
 		rc = scm_cons (node, rc);
 	}
 	return rc;
@@ -781,19 +778,19 @@ do_ret_daily_totals (GttGhtml *ghtml, GttProject *prj)
 
 		/* XXX should use time_t, and srfi-19 to print, and have a type label */
 		/* Print time spent on project this day */
-		qof_print_hours_elapsed_buff (buff, 100, secs, TRUE);
-		node = scm_mem2string (buff, strlen (buff));
+		xxxqof_print_hours_elapsed_buff (buff, 100, secs, TRUE);
+		node = scm_from_locale_string (buff);
 		rpt = scm_cons (node, rpt);
 
 		/* XXX report date should be time_t in the middle of the interval */
 		/* Print date */
 		rptdate = mktime (&tday);
-		qof_print_date_buff (buff, 100, rptdate);
-		node = scm_mem2string (buff, strlen (buff));
+		xxxqof_print_date_buff (buff, 100, rptdate);
+		node = scm_from_locale_string (buff);
 		rpt = scm_cons (node, rpt);
 
 		/* Put a data type in the cdr slot */
-		node = scm_mem2string ("gtt-daily", 9);
+		node = scm_from_locale_string ("gtt-daily");
 		rpt = scm_cons (rpt, node);
 
 		rc = scm_cons (rpt, rc);
@@ -836,7 +833,7 @@ GTT_GETTER##_scm (GttGhtml *ghtml, GttProject *prj)                 \
 {                                                                   \
 	const char * str = GTT_GETTER (prj);                             \
 	if (NULL == str) return SCM_EOL;                                 \
-	return scm_mem2string (str, strlen (str));                       \
+	return scm_from_locale_string (str);                       \
 }                                                                   \
 RET_PROJECT_SIMPLE(RET_FUNC,GTT_GETTER##_scm)
 
@@ -846,17 +843,17 @@ static SCM                                                          \
 GTT_GETTER##_scm (GttGhtml *ghtml, GttProject *prj)                 \
 {                                                                   \
 	long i = GTT_GETTER (prj);                                       \
-	return scm_long2num (i);                                         \
+	return scm_from_long (i);                                         \
 }                                                                   \
 RET_PROJECT_SIMPLE(RET_FUNC,GTT_GETTER##_scm)
 
-                                                                    \
+                                                                    
 #define RET_PROJECT_ULONG(RET_FUNC,GTT_GETTER)                      \
 static SCM                                                          \
 GTT_GETTER##_scm (GttGhtml *ghtml, GttProject *prj)                 \
 {                                                                   \
 	unsigned long i = GTT_GETTER (prj);                              \
-	return scm_ulong2num (i);                                        \
+	return scm_from_ulong (i);                                        \
 }                                                                   \
 RET_PROJECT_SIMPLE(RET_FUNC,GTT_GETTER##_scm)
 
@@ -889,12 +886,12 @@ get_project_title_link_scm (GttGhtml *ghtml, GttProject *prj)
 		g_string_append_printf (str, "<a href=\"gtt:proj:0x%lx\">", (long) prj);
 		g_string_append (str, gtt_project_get_title (prj));
 		g_string_append (str, "</a>");
-		return scm_mem2string (str->str, str->len);
+		return scm_from_locale_string (str->str);
 	}
 	else
 	{
 		const char * str = gtt_project_get_title (prj);
-		return scm_mem2string (str, strlen (str));
+		return scm_from_locale_string (str);
 	}
 }
 
@@ -973,7 +970,7 @@ static SCM                                                          \
 GTT_GETTER##_scm (GttGhtml *ghtml, GttTask *tsk)                    \
 {                                                                   \
 	const char * str = GTT_GETTER (tsk);                             \
-	return scm_mem2string (str, strlen (str));                       \
+	return scm_from_locale_string (str);		 \
 }                                                                   \
                                                                     \
 static SCM                                                          \
@@ -1001,12 +998,12 @@ get_task_memo_scm (GttGhtml *ghtml, GttTask *tsk)
 		g_string_append_printf (str, "<a href=\"gtt:task:0x%lx\">", (long)tsk);
 		g_string_append (str, gtt_task_get_memo (tsk));
 		g_string_append (str, "</a>");
-		return scm_mem2string (str->str, str->len);
+		return scm_from_locale_string (str->str);
 	}
 	else
 	{
 		const char * str = gtt_task_get_memo (tsk);
-		return scm_mem2string (str, strlen (str));
+		return scm_from_locale_string (str);
 	}
 }
 
@@ -1084,7 +1081,24 @@ task_get_time_str_scm (GttGhtml *ghtml, GttTask *tsk)
 	char buff[100];
 
 	task_secs = gtt_task_get_secs_ever(tsk);
-	qof_print_hours_elapsed_buff (buff, 100, task_secs, TRUE);
+	xxxqof_print_hours_elapsed_buff (buff, 100, task_secs, TRUE);
+	return scm_from_locale_string (buff);
+}
+
+static SCM
+task_get_blocktime_str_scm (GttGhtml *ghtml, GttTask *tsk)
+{
+	time_t task_secs;
+	char buff[100];
+	int bill_unit;
+	time_t value;
+
+	task_secs = gtt_task_get_secs_ever(tsk);
+	bill_unit = gtt_task_get_bill_unit(tsk);
+
+	value = (time_t) (lround( ((double) task_secs) / bill_unit ) * bill_unit);
+
+	qof_print_hours_elapsed_buff (buff, 100, value, TRUE);
 	return scm_mem2string (buff, strlen (buff));
 }
 
@@ -1097,11 +1111,11 @@ task_get_earliest_str_scm (GttGhtml *ghtml, GttTask *tsk)
 	size_t len;
 	
 	if (task_date > 0) {
-	    len = qof_print_date_time_buff (buff, 100, task_date);
+	    len = xxxqof_print_date_time_buff (buff, 100, task_date);
 	} else {
         len = g_snprintf(buff, 100, "%s", _("No activity"));
 	}
-	return scm_mem2string (buff, len);
+	return scm_from_locale_string (buff);
 }
 
 static SCM
@@ -1113,11 +1127,11 @@ task_get_latest_str_scm (GttGhtml *ghtml, GttTask *tsk)
 	size_t len;
 
 	if (task_date > 0) {
-	    len = qof_print_date_time_buff (buff, 100, task_date);
+	    len = xxxqof_print_date_time_buff (buff, 100, task_date);
 	} else {
         len = g_snprintf(buff, 100, "%s", _("No activity"));
 	}
-	return scm_mem2string (buff, len);
+	return scm_from_locale_string (buff);
 }
 
 static SCM
@@ -1142,6 +1156,43 @@ task_get_value_str_scm (GttGhtml *ghtml, GttTask *tsk)
 		default: value = 0.0;
 	}
 
+    if (!config_currency_use_locale) {
+      setlocale(LC_MONETARY, "C");
+      setlocale(LC_NUMERIC, "C");
+      snprintf (buff, 100, "%s %.2f", config_currency_symbol, value+0.0049);
+    } else {
+      setlocale(LC_ALL, "");
+      strfmon(buff, 100, "%n", value);
+    }
+
+	return scm_from_locale_string (buff);
+}
+
+static SCM
+task_get_blockvalue_str_scm (GttGhtml *ghtml, GttTask *tsk)
+{
+	time_t task_secs;
+	char buff[100];
+	double value;
+	GttProject *prj;
+	int bill_unit;
+
+	task_secs = gtt_task_get_secs_ever(tsk);
+	bill_unit = gtt_task_get_bill_unit(tsk);
+
+	value = (round( ((double) task_secs) / bill_unit ) * bill_unit) / 3600.0;
+
+	prj = gtt_task_get_parent (tsk);
+	
+	switch (gtt_task_get_billrate (tsk))
+	{
+		case GTT_REGULAR: value *= gtt_project_get_billrate (prj); break;
+		case GTT_OVERTIME: value *= gtt_project_get_overtime_rate (prj); break;
+		case GTT_OVEROVER: value *= gtt_project_get_overover_rate (prj); break;
+		case GTT_FLAT_FEE: value = gtt_project_get_flat_fee (prj); break;
+		default: value = 0.0;
+	}
+
     if (!config_currency_use_locale) {
       setlocale(LC_MONETARY, "C");
       setlocale(LC_NUMERIC, "C");
@@ -1158,9 +1209,11 @@ RET_TASK_STR (ret_task_billstatus,      task_get_billstatus)
 RET_TASK_STR (ret_task_billable,        task_get_billable)
 RET_TASK_STR (ret_task_billrate,        task_get_billrate)
 RET_TASK_SIMPLE (ret_task_time_str,     task_get_time_str)
+RET_TASK_SIMPLE (ret_task_blocktime_str,task_get_blocktime_str)
 RET_TASK_SIMPLE (ret_task_earliest_str, task_get_earliest_str)
 RET_TASK_SIMPLE (ret_task_latest_str,   task_get_latest_str)
 RET_TASK_SIMPLE (ret_task_value_str,    task_get_value_str)
+RET_TASK_SIMPLE (ret_task_blockvalue_str,task_get_blockvalue_str)
 
 /* ============================================================== */
 
@@ -1178,7 +1231,7 @@ static SCM                                                          \
 GTT_GETTER##_scm (GttGhtml *ghtml, GttInterval *ivl)                \
 {                                                                   \
 	const char * str = GTT_GETTER (ivl);                             \
-	return scm_mem2string (str, strlen (str));                       \
+	return scm_from_locale_string (str);		 \
 }                                                                   \
 RET_IVL_SIMPLE(RET_FUNC,GTT_GETTER)
 
@@ -1188,7 +1241,7 @@ static SCM                                                          \
 GTT_GETTER##_scm (GttGhtml *ghtml, GttInterval *ivl)                \
 {                                                                   \
 	unsigned long i = GTT_GETTER (ivl);                              \
-	return scm_ulong2num (i);                                        \
+	return scm_from_ulong (i);                                       \
 }                                                                   \
 RET_IVL_SIMPLE(RET_FUNC,GTT_GETTER)
 
@@ -1204,8 +1257,8 @@ get_ivl_elapsed_str_scm (GttGhtml *ghtml, GttInterval *ivl)
 	time_t elapsed;
 	elapsed = gtt_interval_get_stop (ivl);
 	elapsed -= gtt_interval_get_start (ivl);
-	qof_print_hours_elapsed_buff (buff, 100, elapsed, TRUE);
-	return scm_mem2string (buff, strlen (buff));
+	xxxqof_print_hours_elapsed_buff (buff, 100, elapsed, TRUE);
+	return scm_from_locale_string (buff);
 }
 
 RET_IVL_SIMPLE (ret_ivl_elapsed_str, get_ivl_elapsed_str);
@@ -1217,7 +1270,7 @@ get_ivl_start_stop_common_str_scm (GttGhtml *ghtml, GttInterval *ivl,
 	char buff[100];
 
 	if (prt_date) {
-		qof_print_date_buff (buff, 100, starp);
+		xxxqof_print_date_buff (buff, 100, starp);
 	} else {
         switch (config_time_format) 
         {
@@ -1230,7 +1283,7 @@ get_ivl_start_stop_common_str_scm (GttGhtml *ghtml, GttInterval *ivl,
                 break;
             }
             case TIME_FORMAT_LOCALE: {
-                qof_print_time_buff (buff, 100, starp);
+                xxxqof_print_time_buff (buff, 100, starp);
                 break;
             }
 
@@ -1252,7 +1305,7 @@ get_ivl_start_stop_common_str_scm (GttGhtml *ghtml, GttInterval *ivl,
 		g_string_append (str, "</a>");
 	}
 
-	return scm_mem2string (str->str, str->len);
+	return scm_from_locale_string (str->str);
 }
 
 static SCM
@@ -1270,9 +1323,9 @@ get_ivl_same_day_start_scm (GttGhtml *ghtml, GttInterval *ivl)
 
 	if (0 != prev_stop)
 	{
-		prt_date = qof_is_same_day(start, prev_stop);
+		prt_date = xxxqof_is_same_day(start, prev_stop);
 	}
-	return SCM_BOOL (prt_date);
+	return scm_from_bool (prt_date);
 }
 RET_IVL_SIMPLE (ret_ivl_same_day_start, get_ivl_same_day_start);
 
@@ -1290,9 +1343,9 @@ get_ivl_same_day_stop_scm (GttGhtml *ghtml, GttInterval *ivl)
 	ghtml->last_ivl_time = stop;
 	if (0 != prev_start)
 	{
-		prt_date = qof_is_same_day(prev_start, stop);
+		prt_date = xxxqof_is_same_day(prev_start, stop);
 	}
-	return SCM_BOOL (prt_date);
+	return scm_from_bool (prt_date);
 }
 RET_IVL_SIMPLE (ret_ivl_same_day_stop, get_ivl_same_day_stop);
 
@@ -1333,8 +1386,8 @@ get_ivl_fuzz_str_scm (GttGhtml *ghtml, GttInterval *ivl)
 {
 	char buff[100];
 
-	qof_print_hours_elapsed_buff (buff, 100, gtt_interval_get_fuzz (ivl), TRUE);
-	return scm_mem2string (buff, strlen (buff));
+	xxxqof_print_hours_elapsed_buff (buff, 100, gtt_interval_get_fuzz (ivl), TRUE);
+	return scm_from_locale_string (buff);
 }
 RET_IVL_SIMPLE (ret_ivl_fuzz_str, get_ivl_fuzz_str);
 
@@ -1343,17 +1396,51 @@ RET_IVL_SIMPLE (ret_ivl_fuzz_str, get_ivl_fuzz_str);
 static SCM
 my_catch_handler (void *data, SCM tag, SCM throw_args)
 {
-	printf ("Error: GnoTime caught error during scheme parse\n");
-	if (SCM_SYMBOLP(tag))
-	{
-		char * str  = SCM_SYMBOL_CHARS (tag);
-		printf ("\tScheme error was: %s\n", str);
-	}
-	scm_backtrace();
 
-	SCM fmt = scm_makfrom0str ("~S");
-	SCM s_str = scm_simple_format (SCM_BOOL_F, fmt, SCM_LIST1(throw_args));
-	printf ("\tthrow_args=%s\n", SCM_STRING_CHARS (s_str));
+	printf ("Error: GnoTime caught an error during scheme parse\n");
+
+	SCM the_stack;
+	/* create string port into which we write the error message and
+	   stack. */
+	SCM port = scm_current_output_port();
+	/* throw args seem to be: (FN FORMAT ARGS #f). split the pieces into
+	   local vars. */
+	if (scm_list_p(throw_args) && (scm_length(throw_args) >= 4))
+	{
+		SCM fn = scm_car(throw_args);
+		SCM format = scm_cadr(throw_args);
+		SCM args = scm_caddr(throw_args);
+		SCM other_data = scm_car(scm_cdddr(throw_args));
+		
+		if (fn != SCM_BOOL_F)
+		{ /* display the function name and tag */
+			scm_puts("Function: ", port);
+			scm_display(fn, port);
+			scm_puts(", ", port);
+			scm_display(tag, port);
+			scm_newline(port);
+		}
+
+		if (scm_string_p(format))
+		{ /* conditionally display the error message using format */
+			scm_puts("Error: ", port);
+			scm_display_error_message(format, args, port);
+		}
+		if (other_data != SCM_BOOL_F)
+		{
+			scm_puts("Other Data: ", port);
+			scm_display(other_data, port);
+			scm_newline(port);
+			scm_newline(port);
+		}
+    }
+
+	/* find the stack, and conditionally display it */
+	the_stack = scm_fluid_ref(SCM_CDR(scm_the_last_stack_fluid_var));
+	if (the_stack != SCM_BOOL_F)
+	{
+		scm_display_backtrace(the_stack, port, SCM_UNDEFINED, SCM_UNDEFINED);
+	}
 
 	return SCM_EOL;
 }
@@ -1524,8 +1611,9 @@ gtt_ghtml_display (GttGhtml *ghtml, const char *filepath,
 
 			/* dispatch and handle */
 			scmstart +=5;
-			// scm_c_eval_string (scmstart);
-			gh_eval_str_with_catch (scmstart, my_catch_handler);
+			scm_internal_stack_catch (SCM_BOOL_T, (scm_t_catch_body) scm_c_eval_string,
+				scmstart, (scm_t_catch_handler) my_catch_handler, scmstart);
+
 			start = end;
 			continue;
 		}
@@ -1593,9 +1681,11 @@ register_procs (void)
 	scm_c_define_gsubr("gtt-task-billable",      1, 0, 0, ret_task_billable);
 	scm_c_define_gsubr("gtt-task-billrate",      1, 0, 0, ret_task_billrate);
 	scm_c_define_gsubr("gtt-task-time-str",      1, 0, 0, ret_task_time_str);
+	scm_c_define_gsubr("gtt-task-blocktime-str", 1, 0, 0, ret_task_blocktime_str);
 	scm_c_define_gsubr("gtt-task-earliest-str",  1, 0, 0, ret_task_earliest_str);
 	scm_c_define_gsubr("gtt-task-latest-str",    1, 0, 0, ret_task_latest_str);
 	scm_c_define_gsubr("gtt-task-value-str",     1, 0, 0, ret_task_value_str);
+	scm_c_define_gsubr("gtt-task-blockvalue-str",1, 0, 0, ret_task_blockvalue_str);
 	scm_c_define_gsubr("gtt-task-parent",        1, 0, 0, ret_task_parent);
 
 	scm_c_define_gsubr("gtt-interval-start",     1, 0, 0, ret_ivl_start);
